from matplotlib import pyplot as plt
from cubeplot import draw_cube, draw_face
import numpy as np
from ultralytics import YOLO
import cv2
from tensorflow.keras.models import load_model # type: ignore
from Color_detection import raspberryPi
import os

CAMERA_INDEX = 1

# Initialize cube state
cube_state = {
    'Front': [['white'] * 3 for _ in range(3)],
    'Top': [['yellow'] * 3 for _ in range(3)],
    'Bottom': [['blue'] * 3 for _ in range(3)],
    'Left': [['green'] * 3 for _ in range(3)],
    'Right': [['red'] * 3 for _ in range(3)],
    'Back': [['orange'] * 3 for _ in range(3)],
}

# Define color mapping for matplotlib
color_map = {
    'white': '#FFFFFF',
    'yellow': '#FFFF00',
    'blue': '#0000FF',
    'green': '#00FF00',
    'red': '#FF0000',
    'orange': '#FFA500'
}

# Load the YOLO model
model = YOLO(r"Cube_detection/runs/detect/best/weights/best.pt")

# Open video capture
cap = cv2.VideoCapture(CAMERA_INDEX)
print("Press 'q' to quit.")

model_path = r"Color_detection/rubiks_cube_model.h5"
if not os.path.exists(model_path):
    print("Error: Model file not found.")

print("Loading color detection model...")
model_color = load_model(model_path)

sequence = ['Front', 'Top', 'Bottom', 'Left', 'Right', 'Back']
current_step = 0

# Initialize matplotlib figure


def save_colors(face, colors):
    """Save the detected colors for the given face."""
    cube_state[face] = np.array(colors).reshape(3, 3).tolist()
    
    print(f"Colors for {face} saved:", colors)
# cube_state = {'Front': [['red', 'red', 'white'], ['white', 'white', 'red'], ['red', 'red', 'red']], 
#  'Top': [['white', 'blue', 'white'], ['white', 'white', 'red'], ['white', 'white', 'red']], 
#  'Bottom': [['white', 'blue', 'white'], ['red', 'blue', 'red'], ['red', 'white', 'white']], 
#  'Left': [['white', 'white', 'red'], ['white', 'red', 'white'], ['white', 'white', 'red']], 
#  'Right': [['white', 'red', 'white'], ['red', 'red', 'blue'], ['blue', 'blue', 'blue']], 
#  'Back': [['white', 'yellow', 'red'], ['white', 'yellow', 'blue'], ['blue', 'white', 'blue']]}

# draw_cube(cube_state)

if cube_state ==None or True:
    while True:
        ret, frame = cap.read()
        if not ret:
            print("Failed to capture frame.")
            break

        # Perform detection
        results = model(frame, save=False, conf=0.25)
        annotated_frame = results[0].plot()
        box = results[0].boxes.xyxy

        # Ensure bounding box exists
        if box.size(0) > 0:
            x1, y1, x2, y2 = map(int, box[0])
            cropped_frame = frame[y1:y2, x1:x2]
        else:
            cropped_frame = None

        # Display annotated frame
        cv2.imshow(f"Scanning: {sequence[current_step]} Face", annotated_frame)
        key = cv2.waitKey(1) & 0xFF

        if key == ord('c'):
            if cropped_frame is not None:
                # Display captured frame
                cv2.imshow("Captured Image - Press 'y' to save or 'r' to retake", cropped_frame)

                # Wait for user input
                key_confirm = cv2.waitKey(0) & 0xFF
                cv2.destroyWindow("Captured Image - Press 'y' to save or 'r' to retake")  # Close this window after input

                if key_confirm == ord('y'):
                    print(f"Processing {sequence[current_step]} face...")
                    predicted_colors = raspberryPi.predict_colors(model_color, cropped_frame)
                    colors = raspberryPi.indices_to_colors(predicted_colors)
                    save_colors(sequence[current_step], colors)
                    print(colors)
                    current_step += 1

                    if current_step >= len(sequence):
                        print("All sides scanned successfully!")
                        print("Final Cube Colors:", cube_state)
                        fig = plt.figure()
                        ax = fig.add_subplot(111, projection='3d')
                        draw_cube(cube_state)  # Update visualization
                        break
                elif key_confirm == ord('r'):
                    print("Retaking the picture...")

            else:
                print("No valid region detected. Retake the capture.")

        if key == ord('q'):
            break

# Release resources
cap.release()
cv2.destroyAllWindows()
plt.show()